
import subprocess
import os

def _castorenv(stager, pool):
    env = dict(os.environ)
    env.update({'RFIO_USE_CASTOR_V2':'YES',
               'STAGE_HOST':stager,
               'STAGE_SVCCLASS':pool})
    return env


def checksum(filename, stager, pool, logger=None):

    logger.debug('Fetching checksum from castor for: '
                 + filename)
    nsls = subprocess.Popen(['nsls', '-l', '--checksum', filename],
            stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
            env=_castorenv(stager, pool))
    nsls.wait()
    nslsOut = nsls.stdout.read()
    if logger:
        logger.debug(nslsOut)

    try:
        index = nslsOut.split().index('AD')
        return nslsOut.split()[index+1]
    except ValueError:
        if logger:
            logger.warning('Adler32 checksum not found for '
                           + filename + '. nsls output: '
                           + nslsOut )
        return None


def sizechecksum(filename, stager, pool, logger=None):

    logger.debug('Fetching checksum from castor for: '
                 + filename)
    nsls = subprocess.Popen(['nsls', '-l', '--checksum', filename],
            stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
            env=_castorenv(stager,pool))
    ret = nsls.wait()
    nslsOut = nsls.stdout.read()

    if logger:
        logger.debug(nslsOut)

    size = check_sum = None
    if ret == 0:

        try:
            splitted = nslsOut.split()

            size = int(splitted[4])

            index = splitted.index('AD')
            check_sum = (nslsOut.split()[index+1]).zfill(8)

        except ValueError:
            size = check_sum = None

            if logger:
                logger.warning('Adler32 checksum not found for '
                               + filename + '. nsls output: '
                               + nslsOut)

        except IndexError:
            size = check_sum = None

            if logger:
                logger.warning('Error parsing nsls output '
                               + nslsOut)

    return size, check_sum


def listdir(directory, stager, pool, logger=None):

    nsls = subprocess.Popen(['nsls', directory], stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT, env=_castorenv(stager,pool))
    nslsOut,_ = nsls.communicate()
    ret = nsls.wait()


    if logger:
        logger.debug(nslsOut)

    files = [f for f in nslsOut.split('\n') if f]

    return ((ret == 0), files)


def migrated(filename, stager, pool, logger=None):

    nsls = subprocess.Popen(['nsls', '-l', filename], stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT, env=_castorenv(stager,pool))
    ret = nsls.wait()
    nslsOut = nsls.stdout.read()

    if logger:
        logger.debug(nslsOut)

    return (ret == 0 and 'm' in nslsOut.split(' ')[0])


def mkdir(directory, stager, pool, logger=None):

    nsmkdir = subprocess.Popen(['nsmkdir', '-p', directory],
            stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
            env=_castorenv(stager,pool))
    ret = nsmkdir.wait()

    if logger:
        logger.debug(nsmkdir.stdout.read())

    return ret


def remove(dstfile, stager, pool, logger=None):

    rfrm = subprocess.Popen(['rfrm', dstfile], stdout=subprocess.PIPE,
                 stderr=subprocess.STDOUT, env=_castorenv(stager,pool))
    ret = rfrm.wait()

    if logger:
        logger.debug(rfrm.stdout.read())

    return ret


def backgroundcopy(srcfile, dstfile, stager, pool, logger=None):
    del logger # unused args

    rfcp = subprocess.Popen(['rfcp', srcfile, dstfile], stdout=subprocess.PIPE,
                 stderr=subprocess.STDOUT, env=_castorenv(stager,pool))

    return rfcp, rfcp.pid


def copystate(proc):

    return proc.poll()


def copystdout(proc):

    return proc.stdout.read()


if __name__=='__main__':
    import sys
    from os.path import dirname, abspath, join

    # add /../../CastorScript/Script to path so imports keeps working
    SCRIPT_DIR = abspath(join(dirname(__file__), '..','..'))
    sys.path.append(SCRIPT_DIR)
    
    import cs.Tools.storagetester as storagetester
    storagetester.execute(sys.modules[__name__])
