"""
Provides EOS backend functionalities.
Tape migration is not supported by EOS.
"""
import subprocess
import os
import sys

# if this file runs as a script instead of as a module.
if __name__=='__main__':
    from os.path import dirname, abspath, join

    # add /../../CastorScript/Script to path so imports keeps working
    SCRIPT_DIR = abspath(join(dirname(__file__), '..','..'))
    sys.path.append(SCRIPT_DIR)

import cs.StorageBackends.castorstorage as castorstorage

## Global object storing configuration parameters ##
## copy_extra_params --> extra parameters appended to copy operations
config_dict = {}

def init(d):

    global config_dict
    config_dict = d


def sizechecksum(filename, stager, pool, logger=None):
    del pool # unused args

    if logger:
        logger.debug('Fetching checksum from eos for: '
                     + filename)

    # SIZE
    # Query for generalized stats
    cmd=['xrdfs', '%s' % (stager),
         'stat','%s' % (filename)]

    xrdfs = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    ret = xrdfs.wait()
    out = xrdfs.stdout.read()

    # Find and get the size, log exception if any
    size = None
    if not ret:
        try:
            for line in out.split('\n'):
                if 'Size:' in line:
                    size = int(line.split()[1])

        except (KeyError,AttributeError,ValueError):
            if logger:
                logger.warning('Error parsing size for ' + filename \
                               + ' Command: ' + '%s' % cmd \
                               + ' Output: ' + out,
                               exc_info=True)
    else:
        if logger:
            logger.warning('Size not found for '
                           + filename + ': '
                           + out)

    # CHECKSUM
    # Query for checksum
    cmd=['xrdfs', '%s' % (stager), 'query',
         'checksum','%s' % (filename)]

    xrdfs = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    ret = xrdfs.wait()
    out = xrdfs.stdout.read()

    # Get Checksum, log exception if any
    checksum = None
    if not ret:
        try:
            checksum = out.split()[1]
        except (KeyError, AttributeError, ValueError, IndexError):
            if logger:
                logger.warning('Error parsing checksum for ' \
                               + filename \
                               + ' Command: ' + '%s' % cmd \
                               + ' Output: ' + out,
                               exc_info=True)

    else:
        if logger:
            logger.warning('Checksum not found for '
                           + filename + ': '
                           + out)



    return size,checksum


def remove(dstfile, stager, pool, logger=None):
    del pool # unused args

    xrdfs = subprocess.Popen(['xrdfs',stager,'rm',dstfile], stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT)
    ret = xrdfs.wait()

    if logger:
        logger.debug(xrdfs.stdout.read())

    return ret


def backgroundcopy(srcfile, dstfile, stager, pool, logger=None):
    del logger #unused args

    dstfile = 'root://%s/%s' % (stager, dstfile)
    cmd = ['xrdcp', '-f', '--nopbar', srcfile, dstfile]

    try:
        cmd.append(config_dict['copy_extra_params'])
    except KeyError:
        pass

    if pool:
        cmd.append('-ODsvcClass=%s' % pool)

    # Disable default XRootD's writing recovery policy
    modified_env = os.environ.copy()
    if 'XRD_WRITERECOVERY' in config_dict:
        modified_env['XRD_WRITERECOVERY'] = str(config_dict['XRD_WRITERECOVERY'])
    if 'XRD_STREAMTIMEOUT' in config_dict:
        modified_env['XRD_STREAMTIMEOUT'] = str(config_dict['XRD_STREAMTIMEOUT'])

    xrdcp = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                             stderr=subprocess.STDOUT, env=modified_env)

    return xrdcp, xrdcp.pid


def mkdir(directory, stager, pool, logger=None):
    """
    In EOS directories are automagically created during copy, so this is just a dummy method.
    """
    del directory, stager, pool, logger # unused args
    return 0


def migrated(filename, stager, pool, logger=None):
    del filename, stager, pool, logger # unused args
    return False


copystate = castorstorage.copystate
copystdout = castorstorage.copystdout

if __name__=='__main__':
    import cs.Tools.storagetester as storagetester
    storagetester.execute(sys.modules[__name__])